<?php
/**
 * Javascript code editor: "EditArea"
 *
 * EditArea is created by Christophe Dolivet and released under "LGPL", 
 * "Apache" and "BSD" licenses. The LGPL license integration into Pacer Edition 
 * was choosen.
 *
 * LICENSE: GNU Lesser General Public License 3.0
 * 
 * @author		Christophe Dolivet (EditArea), JCWEBDEN  (Modification into PE wrapper)
 * @copyright	(c) 2009
 * @license		http://www.gnu.org/copyleft/lesser.html
 * @version		0.8.1.1 
 * @platform	Pacer Edition
 
  The Pacer Edition Project <http://www.jcwebden.com/>
 Copyright (C) 2009, John P. and Stefano R.

 Pacer Edition is free and Commercial software; you can redistribute it
 and/or modify it under the terms of the GNU General Public License as
 published by the Free Software Foundation; either version 2 of the License,
 or (at your option) any later version.

 Pacer Edition is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Website Baker; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Website Baker Project <http://www.websitebaker.org/>
 Copyright (C) 2004-2009, Ryan Djurovich
*/

if (!function_exists('registerEditArea')) {
	function registerEditArea($id = 'code_area', $syntax = 'php', $syntax_selection = true
		, $allow_resize = 'no', $allow_toggle = true, $start_highlight = true
		, $min_width = 600, $min_height = 300, $toolbar = 'default')
	{ 
		// set default toolbar if no user defined was specified
		if ($toolbar == 'default') {
			$toolbar = 'search, fullscreen, |, undo, redo, |, select_font, |, syntax_selection, |, highlight, reset_highlight, |, help';
			$toolbar = (!$syntax_selection) ? str_replace('|, syntax_selection,', '', $toolbar) : $toolbar;
		}
	
		// check if used Pacer Edition backend language is supported by EditArea
		$language = 'en';
		if (defined('LANGUAGE') && file_exists(dirname(__FILE__) . '/langs/' . strtolower(LANGUAGE) . '.js')) {
			$language = strtolower(LANGUAGE);
		}

		// check if highlight syntax is supported by edit_area
		$syntax = in_array($syntax, array('css', 'html', 'js', 'php', 'xml')) ? $syntax : 'php';

		// check if resize option is supported by edit_area
		$allow_resize = in_array($allow_resize, array('no', 'both', 'x', 'y')) ? $allow_resize : 'no';

		// return the Javascript code
		$result = <<< EOT
		<script language="javascript" type="text/javascript">
			editAreaLoader.init({
				id: 				'$id'
				,start_highlight:	$start_highlight
				,syntax:			'$syntax'
				,min_width:			$min_width
				,min_height:		$min_height
				
				,show_line_colors: true

				,allow_resize: 		'$allow_resize'
				,allow_toggle: 		$allow_toggle
				,toolbar: 			'$toolbar'
				,language:			'$language'
			});
		</script>
EOT;
		return $result;	
	}
}

if (!function_exists('getEditAreaSyntax')) {
	function getEditAreaSyntax($file) 
	{
		// returns the highlight scheme for edit_area
		$syntax = 'php';
		if (is_readable($file)) {
			// extract file extension
			$file_info = pathinfo($file);
		
			switch ($file_info['extension']) {
				case 'htm': case 'html': case 'htt':
					$syntax = 'html';
	  				break;

	 			case 'css':
					$syntax = 'css';
	  				break;

				case 'js':
					$syntax = 'js';
					break;

				case 'xml':
					$syntax = 'xml';
					break;

	 			case 'php': case 'php3': case 'php4': case 'php5':
					$syntax = 'php';
	  				break;

				default:
					$syntax = 'php';
					break;
			}
		}
		return $syntax ;
	}
}

?>