<?php

/*
 * $Id$
 * The Following license and copyright information must not be removed
 *
 * Software Name: The Pacer Edition CMS
 *    Build Date: July 31, 2009
 *    Build Time: 19:00:00
 *     File Name: pacer_error_handler.php
 *           URL: http://www.jcwebden.com
 *    PE License: Copyright 2009 The Pacer Edition (JCWEBDEN.COM)
 *                <http://www.jcwebden.com/online/legal/copyright.php>
 *    WB License: GNU General Public License
 *                <http://www.gnu.org/licenses/gpl-3.0-standalone.html>
 *
 *
 * Copyright (C) 2009, JCWEBDEN.COM (DARKSTAR INCORPORATED)
 * 
 * The Pacer Edition CMS is provided as free open source software 
 * as well as providing commercial applications within the scope of 
 * The Pacer Edition CMS software;
 * You can redistribute The Pacer Edition CMS and/or modify it under
 * the terms of the GNU General Public License as published by the
 * Free Software Foundation; version 3 of the License.
 * Though the scope of providing commercial add-ons is bound by the
 * copyright terms laid out at
 * <http://www.jcwebden.com/online/legal/copyright.php>
 *
 * The Pacer Edition CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * A copy of the GNU General Public License for Website Baker Core
 * is provided to you within this script named(GNULICENSE.txt)
 * it also made available by going to
 * <http://www.gnu.org/licenses/gpl-3.0-standalone.html>
 *
 * You should have received a copy of the GNU General Public License
 * along with The Pacer Edition CMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * The Pacer Edition CMS <http://www.jcwebden.com/>
 * Copyright (C) 2009, The Pacer Edition (JCWEBDEN.COM)
 *
 * The following must accompany any and all parts of this script
 * Website Baker Project <http://www.websitebaker.org/>
 * Copyright (C) 2004-2009, Ryan Djurovich
 *
 */
/**
 * @since PHP 5
 */
if(!defined('E_STRICT'))			define('E_STRICT', 2048);

/**
 * @since PHP 5.2.0
 */
if(!defined('E_RECOVERABLE_ERROR'))	define('E_RECOVERABLE_ERROR', 4096);

/**
 * @since PHP 5.3.0
 */
if(!defined('E_DEPRECATED'))		define('E_DEPRECATED', 8192);
if(!defined('E_USER_DEPRECATED'))	define('E_USER_DEPRECATED', 16384);


if (!function_exists('Pacer_Error_Handler')) {
function Pacer_Error_Handler($errno, $errstr, $errfile, $errline){

	$replevel = error_reporting();

	// if the error has been suppressed by the @ we don't handle the error
	if( $replevel == 0 || $errno == 0)
		return true;
    
	// we shall remain quiet.
	if( ( $errno & $replevel ) != $errno )
		return true;

	// Suppress errors we dont want to report
	$formula=@unserialize(ER_FORMULA);
	if (is_array($formula) && array_key_exists($errno,array_flip($formula)))
		return true;

	// If error is E_STRICT and it comes from pear... ignore it	
	if (($errno & E_STRICT) && (preg_match('|^'.str_replace('\\','/',PACER_PATH).'/pear/.*$|i' ,str_replace('\\','/',$errfile))))
		return true;

	switch($errno){
		case E_ERROR:
		case E_PARSE:
		case E_CORE_ERROR:
		case E_CORE_WARNING:
		case E_COMPILE_ERROR:
		case E_COMPILE_WARNING:
		case E_USER_ERROR:
			$type = 'Fatal error';
        	$fatal = true;
			break;
		
		case E_WARNING:
		case E_NOTICE:
		case E_USER_WARNING:
		case E_USER_NOTICE:
		case E_STRICT:
		case E_RECOVERABLE_ERROR:
		case E_DEPRECATED:
		case E_USER_DEPRECATED:
		default:
			// do not exit
			$type = 'Warning';
    	    $fatal = false;
			break;
	}
		
	// Log error
	try {
		global $Pacer;
		$Pacer->LogError($errstr . ' in ' . $errfile . ' at line ' . $errline);
	} catch(Exception $e) {
		// in case the error occurs before the logger creation, we simply display it
		print("<pre>$errstr \nin '$errfile' at line $errline\n</pre>");
	}

	// drain the output...
	while (ob_get_level()) {
		ob_end_flush();
   	}	
	
	if (ER_BACKTRACE){
		$trace = array_reverse(debug_backtrace());
    	array_pop($trace);
	}
	// Print error
	if(php_sapi_name() == 'cli') {
    	echo ((ER_BACKTRACE)?'Backtrace from ':'') . $type . ': ['.$errno.'] \'' . $errstr . '\' at ' . $errfile . ' ' . $errline . ':' . "\n";
		if (ER_BACKTRACE) {
   			foreach($trace as $item)
           		echo '  ' . (isset($item['file']) ? $item['file'] : '<unknown file>') . ' ' . (isset($item['line']) ? $item['line'] : '<unknown line>') . ' calling ' . $item['function'] . '()' . "\n";
		}
	} else {
       	echo '<p class="error_backtrace">';
    	echo ((ER_BACKTRACE)?'Backtrace from ' :'') . $type . ': ['.$errno.'] \'' . $errstr . '\' at ' . $errfile . ' ' . $errline . ':';
   		if (ER_BACKTRACE) {
			echo '<ol>';
       		foreach($trace as $item)
       	   		echo '<li>' . (isset($item['file']) ? $item['file'] : '<unknown file>') . ' ' . (isset($item['line']) ? $item['line'] : '<unknown line>') . ' calling ' . $item['function'] . '()</li>';
    		echo '</ol>';
		}
    	echo '</p>';
 

	if($fatal)
		die();
	}
}
}
?>