<?php

require_once(PACER_PATH.'/pacer.php');

if (!class_exists('Pacer_Installation_Watchdog')) {
	class Pacer_Installation_Watchdog extends Pacer {
		private $debug = true;
		private $list_time;
		private $last_time;
		private $max_lists = 3;
		private $skip_files = array();
		private $skip_regex = array();
		
		var $compare_msg = array();
		
		function __construct($send_email=true,$report_errors=false,$show_results=false) {
			parent::__construct();
			$this->list_time = time();
			$this->last_time = $this->database->get_one("SELECT checked_when FROM `".TABLE_PREFIX."pacer_installation_watchdog` WHERE checked_when < '".$this->list_time."' ORDER BY checked_when DESC LIMIT 1");
			
			$this->create_skip_lists();
					
			$this->clean_list($this->max_lists);
			$this->create_list(PE_PATH);
			
			$this->compare_list($show_results);
			
			if ($send_email) return $this->send_report($report_errors);
			return true;
	
		}
		//ALTER TABLE `wb_pacer_installation_watchdog` CHANGE `id` `id` INT( 11 ) NOT NULL 
		//ALTER TABLE `wb_pacer_installation_watchdog` DROP PRIMARY KEY 
		//ALTER TABLE `wb_pacer_installation_watchdog` DROP `id` 
		//ALTER TABLE `pacer`.`wb_pacer_installation_watchdog` DROP INDEX `checked_when` , ADD PRIMARY KEY ( `checked_when` , `folder` ( 250 ) , `file` ( 60 ) ) 
		
		private function clean_list($max_lists){
			$list_times = $this->database->queryAll("SELECT checked_when FROM `".TABLE_PREFIX."pacer_installation_watchdog` WHERE checked_when < '".$this->list_time."' GROUP BY checked_when ORDER BY checked_when DESC");
			if (count($list_times)>$max_lists){		
				$this->database->query("DELETE FROM `".TABLE_PREFIX."pacer_installation_watchdog` WHERE checked_when < '".$list_times[$max_lists]['checked_when']."'");
			}
		}
			
		private function create_list($directory){
			//glob all...
				
			$fd = str_replace(PE_PATH,'',$directory);
			// Function to open a directory
			$dir = dir($directory);
			while (false !== $entry = $dir->read()) {
				// Skip pointers
				if(($entry=='.') || ($entry=='..')) continue;
				$fn = $fd.'/'.$entry;
				$file = $directory.'/'.$entry;
				// Skip files
				if (in_array($fn,$this->skip_files)) continue;
				if ($this->check_regex($fn)) continue;
				
				if (is_dir($file)) {
					$this->create_list($file);	
				} else {					
					$vars=array(); 				$values=array();
					$vars[]	= "checked_when";	$values[] = $this->list_time;
				
					$vars[]	= "folder";			$values[] = addslashes($fd);
					$vars[]	= "file";			$values[] = addslashes($entry);
				
					$vars[]	= "size";			$values[] = filesize($file);
					$vars[]	= "filectime";		$values[] = filectime($file);
					$vars[]	= "fileatime";		$values[] = fileatime($file);
					$vars[]	= "filemtime";		$values[] = filemtime($file);
				
					$vars[]	= "permissions";	$values[] = $this->human_file_perms('',$file);
					$vars[]	= "checksum";		$values[] = addslashes(md5_file($file));
					$vars[]	= "group";			$values[] = ((function_exists('posix_getgrgid')) ?posix_getgrgid(filegroup($file)):'');	
					$vars[]	= "owner";			$values[] = ((function_exists('posix_getpwuid')) ? posix_getpwuid(fileowner($file)):'');
					
					//crc64()	
				
					$this->database->query("INSERT INTO `".TABLE_PREFIX."pacer_installation_watchdog` (`".implode("`,`",$vars)."`) VALUES ('".implode("','",$values)."')");
				}
			}		
		}
		
		private function check_regex($fn) {
			$ret = false;
			foreach($this->skip_regex as $regex){
				if (preg_match($regex,$fn)) {
					$ret = true;
					break;
				}
			}
			return $ret;
		}
		
		private function compare_list($show_results) {
			$this->compare_msg = array();
			if ($this->last_time) {
				$rows = $this->database->queryAll("SELECT * FROM `".TABLE_PREFIX."pacer_installation_watchdog` WHERE checked_when = '".$this->list_time."'");
				foreach($rows as $row){
					$prev_row = $this->database->queryRow("SELECT * FROM `".TABLE_PREFIX."pacer_installation_watchdog` WHERE checked_when = '".$this->last_time."' AND folder = '".addslashes($row['folder'])."' AND file = '".addslashes($row['file'])."' LIMIT 1");
					$mfile = array();
					if (count($prev_row)){ //compare with prev file
						if ($row['size']!==$prev_row['size']){
							$mfile[]="Size: ".$row['size']."!==".$prev_row['size'];									
						} elseif ($row['checksum']!==$prev_row['checksum']){
							$mfile[]="Checksum: ".$row['checksum']."!==".$prev_row['checksum'];									
						}
						if ($row['permissions']!==$prev_row['permissions']){
							$mfile[]="Permissions: ".$row['permissions']."!==".$prev_row['permissions'];									
						}
						if ($row['owner']!==$prev_row['owner']){
							$mfile[]="Owner: ".$row['owner']."!==".$prev_row['owner'];									
						}if ($row['group']!==$prev_row['group']){
							$mfile[]="Group: ".$row['group']."!==".$prev_row['group'];									
						}
						if ($row['filemtime']!==$prev_row['filemtime']){
							$mfile[]="Modified: ".date(DATE_FORMAT.' '.TIME_FORMAT,$row['filemtime'])."!==".date(DATE_FORMAT.' '.TIME_FORMAT,$prev_row['filemtime']);									
						}	
					} else { // new file
						$mfile[]="New! - Created: ".date(DATE_FORMAT.' '.TIME_FORMAT,$row['filectime']);				
					}
					if (count($mfile)>0){
						$mfile="File ".$row['folder'].'/'.$row['file']."\n".'-['.implode("]\n-[",$mfile).']';
						$this->compare_msg[]=$mfile;
						
						if($show_results) echo $mfile."\n";
					}
				}
			}
		}
		
		private function send_report($report_errors) {
			if (count($this->compare_msg)){ // send report
				$email_from 	= $this->pacer_settings['mail_default_sender'];	// FROM: set default name
				$email_fromname = $this->pacer_settings['mail_default_sendername'];// FROM NAME: set default name
				//$email_from = SERVER_EMAIL;
				//$email_fromname = wbmailer_default_sendername;
				// admin email
				$email_to = $this->database->get_one(sprintf("SELECT `email` FROM `%susers` WHERE `user_id` = '%s'",TABLE_PREFIX,1));
				$email_subject = 'Pacer Installaton Watchdog - Files changed since last check on '.date(DATE_FORMAT.' '.TIME_FORMAT,$this->last_time);	
				$email_body = implode("\n",$this->compare_msg);
				
				$er=error_reporting();
				error_reporting(0);
				// mail to 
				if(!$this->Send($email_from,$email_to,$email_subject,$email_body,$email_fromname)) {
					error_reporting($er);
					
					$this->LogEvent($this->Translate('Mail Routine Error')."\n".$email_subject."\n".$email_body);
			    	if($report_errors) $this->print_error($this->Translate("Report of changed files cannod be emailed to administrator!"),'','stiky');

					return false;
				}
				error_reporting($er);
				
				$this->LogEvent($email_subject."\n".$email_body);
				if($report_errors) $this->print_success($this->Translate("Report of changed files emailed to administrator"),'','stiky');
				return true;
			}
	    	if($report_errors) $this->print_success($this->Translate("No changed files to report"),'','stiky');
			return true;
		}
		
		private function create_skip_lists() {
			
			$this->skip_regex = array();
			$this->skip_files = array();
			
			$tf = str_replace(PE_PATH,'',(($this->pacer_settings['cache_folder']) ? PE_PATH.$this->pacer_settings['cache_folder']:TEMP_PATH).'/cached_pages');
			
			foreach(array(str_replace('\\','/',str_replace(PE_PATH,'',$this->log_filename)),$tf,'/temp/cached_images','/temp/URI','/temp/CSS','/temp/HTML') as $k){
				$this->skip_files[$k] = $k;
			}
			
			if ($arr=preg_split("/[;,\n\r]+/", $this->pacer_settings['installation_watchdog_exclude'])){
				foreach($arr as $entry){
					$p = substr($entry,0,1);
					switch($p){
						case '|':
							$this->skip_regex[$entry] = $entry;
							break;
						default:
							$this->skip_files[$entry] = $entry;
					}
				}
			}			
		}
		
		private function do_debug($msg) {
			if ($this->debug) $this->LogEvent($msg);
		}
	}
}
?>